#include <wx/wxprec.h>
#ifndef WX_PRECOMP
   #include <wx/wx.h>
#endif

#include "BoardPanel.h"
//#include "BoardData.h"

BoardPanel::BoardPanel(wxFrame *parent/*, BoardData &data*/)
    : wxPanel(parent, wxID_ANY)/*, model(data)*/, currentsquare(nullptr)
{
    CreateBoard();
}

void BoardPanel::CreateBoard()
{
	// sizers for the layout of bricks and buttons
	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);	// overall layout
	// 8 rows and 8 columns with fixed height and width
	wxGridSizer *gs = new wxGridSizer(8, 8, 1, 1);
	wxBoxSizer *hbox2 = new wxBoxSizer(wxHORIZONTAL);		// button commands

	// Define font and font size for the text in the squares
	// Define valid input values for the text in the squares
	csCharCtrl::Initialize(wxT("bBwW"), sz);

	// Create the squares
    for (int i = 0; i < 64; i++) {
        m_square[i] = CreateSquare(this, i+1, i/8, i%8);
		// install an event handler for Set Focus event for the square
		gs->Add(m_square[i], 0, wxEXPAND);
    }
    Bind(wxEVT_CHILD_FOCUS, &BoardPanel::OnChildFocus, this);

	// buttons commands
	// Close command
	wxButton *btnclose = new wxButton(this, ID_Close, wxT("Close"));
    btnclose->Bind(wxEVT_COMMAND_BUTTON_CLICKED, &BoardPanel::OnClose, this);

	vbox->Add(gs, 0, wxALIGN_CENTER | wxALL, 20);

	hbox2->Add(btnclose, 1, wxEXPAND);
	vbox->Add(hbox2, 0, wxALIGN_RIGHT | wxALL, 20);

	SetSizer(vbox);

}

csCharCtrl *BoardPanel::CreateSquare(wxPanel *panel, int nbr, int row, int col)
{
	// Is the square black or white
	bool isblack = (row+col)%2 == 1;

	// Create the text input field - white squares are read only
	csCharCtrl *field = new csCharCtrl(panel, ID_Square_Start + row*8 + col, !isblack);

	// Color the squares "black" or white
	field->SetBackgroundColour(isblack ? *wxLIGHT_GREY : *wxWHITE);

    return field;
}

// Input focus has changed
void BoardPanel::OnChildFocus(wxChildFocusEvent& event)
{
	if ((currentsquare != nullptr) && currentsquare->IsModified()) {
		char charvalue;
		if (currentsquare->GetCharValue(charvalue)) {
			int sqnbr = currentsquare->GetId() - ID_Square_Start;
			int row = sqnbr / 8;
			int col = sqnbr % 8;
			// update model with new value
			// wxLogDebug(wxString::Format("OnChildFocus: row %d, col %d, char '%c'", row, col, charvalue));
// TO DO: Update the model with new value
			// model->SetXX(row, col, charvalue);
		}
	}
	// new current text input field (nullptr if not a csCharCtrl)
	currentsquare = dynamic_cast<csCharCtrl *>(wxWindow::FindFocus());
}

void BoardPanel::OnCheck(wxCommandEvent& WXUNUSED(event))
{
}

void BoardPanel::OnReset(wxCommandEvent& WXUNUSED(event))
{
}

void BoardPanel::OnClose(wxCommandEvent& WXUNUSED(event))
{
	// Assume that the parent window is the main frame window
	GetParent()->Close(true);
}

// Display text in the status bar
void BoardPanel::SetStatusText(const wxString &text, int number) {
	wxFrame * frame = dynamic_cast<wxFrame*> (GetParent());
	if (frame != nullptr)
		frame->SetStatusText(text, number);
}

